﻿class PlayerAvatar extends MovieClip
{
	var startX:Number;
	var startY:Number;
	var gravitydir:Number;
	
	var gravity:Number;
	var maxX;
	var maxY;
	var keyIsDown;
	var dir:Vect;
	var velocity:Vect;
	var radius:Number;
	var pnr:Number;
	var maxbounce:Number;
	
	var power:Number;
	var normalPower:Number;
	var maxPower:Number;
	var powerBuildup:Number;
	var decayMultiplier:Number;
	var maxSpeed:Number;
	
	public function onForwardDown()
	{
		keyIsDown = true;
	}

	
	public function onForwardUp()
	{
		keyIsDown = false;
	}
	
	public function restart()
	{
		_x = startX;
		_y = startY;
		velocity.x = 0;
		velocity.y = 0;
	}
	
	public function SetXML(xmlCont:XML)
	{
		var values:Array = xmlCont.firstChild.childNodes;
		startX = parseInt(values[0].firstChild);
		startY = parseInt(values[1].firstChild);
		_x = startX;
		_y = startY;
	}
	
	public function SetPlayerNumber(nr:Number)
	{
		pnr = nr;
		if ( nr == 1 )
		{
			gravitydir = 1;
		}
		else
		{
			gravitydir = -1;
		}
	}
	
	public function onLoad()
	{
		Key.addListener(this);
		
		maxX = 800;
		maxY = 600;
		keyIsDown = false;
		//_rotation = 0;
		
		gravity = 0.2;
		
		dir = new Vect(0, -1);
		velocity = new Vect(0, 0);
		radius = 50;
		
		power = 0;
		normalPower = 0.25;
		maxPower = 15;
		powerBuildup = 0.4;
		
		decayMultiplier = 0.97;
		maxbounce = 5.0;
		maxSpeed = 5.0;
	}
	
	public function SetPhysics(xmlCont:XML)
	{
		//<gravity>0.2</gravity>
		//<power>0.25</power>
		//<decay>0.03</decay>
		//<playerRadius>10<playerRadius>
		var values:Array = xmlCont.firstChild.childNodes;
		gravity = parseFloat(values[0].firstChild);
		normalPower = parseFloat(values[1].firstChild);
		decayMultiplier = 1.0 - parseFloat(values[2].firstChild);
		radius= parseFloat(values[3].firstChild);
		maxbounce = parseFloat(values[4].firstChild);
		maxSpeed = parseFloat(values[5].firstChild);
	}
	
	public function Update()
	{
		if ( _root._currentframe != 3 )
		{
			return;
		}
		
		doPhysics();
	}
	
	private function doMove(pathDir:Vect)
	{
		var addVect = new Vect(pathDir.x, pathDir.y);
		
		addVect.multiply(power);
		velocity.addVector(addVect);
		power = normalPower;

	}
	
	private function doPhysics()
	{
		if ( _x < 0 || _y < 0 || _x > Stage.width || _y > Stage.height )
		{
			var frame:Number = 6-pnr;
			trace(_x+" y:"+_y);
			_root.gotoAndStop(frame);
		}
		
		velocity.y += gravitydir * gravity;
		velocity.multiply(decayMultiplier);
		_x += velocity.x;
		_y += velocity.y;
		
		if(velocity.GetLength() > maxSpeed)
		{
			velocity.multiply(maxSpeed/velocity.GetLength());
		}
		
		if(Accessor.GetCurrentAlternateTrack()._x < Stage.width)
		{
			doTrackHandling(Accessor.GetCurrentAlternateTrack());
		}
		if(Accessor.GetCurrentTrack()._x < Stage.width)
		{
			doTrackHandling(Accessor.GetCurrentTrack());
		}
	}
	
	private function doTrackHandling(track:Track)
	{
		var hitSegment:TrackSegment = TrackSegment(track.intersect(_x, _y, radius));
		
		if ( hitSegment != undefined )
		{
			if(keyIsDown)
			{
				doMove(hitSegment.GetDir());
			}
			
			var hitNormal:Vect = Vect(hitSegment.GetLeftNormal());
			var inDir = velocity.dot(hitNormal);
			
			//trace("velx: " + velocity.x + " vely: " + velocity.y + " normx: " + hitNormal.x + " normy: " + hitNormal.y);
			
			var pushOut = hitSegment.getDistance(_x , _y);
			if ( Math.abs(pushOut) < radius )
			{
				if ( inDir < 0)
				{
					pushOut = - Math.abs(radius-Math.abs(pushOut));
				}
				else
				{
					pushOut = Math.abs(radius-Math.abs(pushOut));
				}
				
				_x -= pushOut * hitNormal.x;
				_y -= pushOut * hitNormal.y;
			}
			
			var bnz:Number = Math.min(maxbounce, Math.abs(inDir));
			
			if ( inDir < 0 )
			{
				inDir = - bnz;
			}
			else
			{
				inDir = bnz;
			}
			
			velocity.x-= inDir * hitNormal.x;
			velocity.y-= inDir * hitNormal.y;
		}
	}
}